package com.ufund.api.ufundapi.controller;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.ufund.api.ufundapi.model.*;
import com.ufund.api.ufundapi.persistence.CupboardDAO;
import com.ufund.api.ufundapi.persistence.FundingBasketDAO;

/**
 * Handles the REST API requests for the Basket resource
 * <p>
 * {@literal @}RestController Spring annotation identifies this class as a REST
 * API method handler
 * to the Spring framework
 */

@RestController
@RequestMapping("basket")
public class FundingBasketController {
	private static final Logger LOG = Logger.getLogger(FundingBasketController.class.getName());
	private FundingBasketDAO basketDao;
	private CupboardDAO needDao;

	/**
	 * Creates a REST API controller to respond to requests
	 *
	 * @param basketDao The {@link FundingBasketDAO Basket Data Access Object} to
	 *                  perform CRUD operations
	 *                  <br>
	 *                  This dependency is injected by Spring Framework
	 * @param needDao   The {@link CupboardDAO Cupboard Data Access Object} to
	 *                  perform CRUD operations
	 *                  <br>
	 *                  This dependency is injected by Spring Framework
	 */
	public FundingBasketController(FundingBasketDAO basketDao, CupboardDAO needDao) {
		this.basketDao = basketDao;
		this.needDao = needDao;
	}

	/**
	 * Adds a {@link Need need} with the provided need ID
	 *
	 * @param username - The username of the user to add the {@link Need need} to
	 * @param id       - The {@link Need need} ID to add
	 *
	 * @return ResponseEntity with the added {@link Need need} object and HTTP
	 *         status of CREATED<br>
	 *         ResponseEntity with HTTP status of CONFLICT if {@link Need need}
	 *         ID already exists<br>
	 *         ResponseEntity with HttpStatus of INTERNAL_SERVER_ERROR otherwise
	 */
	@PostMapping("/{username}")
	public ResponseEntity<Need> addNeed(@PathVariable String username, @RequestBody Integer id) {
		LOG.info("POST /basket/" + username + " " + id);
		try {
			if (basketDao.addNeed(username, id)) {
				Need need = needDao.getNeed(id);
				return new ResponseEntity<Need>(need, HttpStatus.CREATED);
			} else
				return new ResponseEntity<>(HttpStatus.CONFLICT);
		} catch (IOException e) {
			LOG.log(Level.SEVERE, e.getLocalizedMessage());
			return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}

	/**
	 * Get the {@link Need need} from the basket by its ID
	 *
	 * @param username - The username of the user to get the {@link Need need} for
	 * @param id       - The {@link Need need} ID to get
	 *
	 * @return ResponseEntity with the {@link Need need} object and HTTP status of
	 *         OK<br>
	 *         ResponseEntity with HTTP status of NOT_FOUND if {@link Need need} ID
	 *         does not exist<br>
	 *         ResponseEntity with HttpStatus of INTERNAL_SERVER_ERROR otherwise
	 */
	@GetMapping("/{username}/{id}")
	public ResponseEntity<Need> getNeed(@PathVariable String username, @PathVariable int id) {
		LOG.info("GET /basket/" + username + "/" + id);
		try {
			if (basketDao.getNeed(username, id)) {
				Need need = needDao.getNeed(id);
				return new ResponseEntity<Need>(need, HttpStatus.OK);
			} else {
				return new ResponseEntity<>(HttpStatus.NOT_FOUND);
			}
		} catch (IOException e) {
			LOG.log(Level.SEVERE, e.getLocalizedMessage());
			return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}

	/**
	 * Get the {@link Need[] needs} IDs from the basket
	 *
	 * @param username - The username of the user to get the {@link Need needs} for
	 *
	 * @return ResponseEntity with {@link Need[] needs} objects and HTTP
	 *         status of OK
	 *         ResponseEntity with HttpStatus of INTERNAL_SERVER_ERROR otherwise
	 */
	@GetMapping("{username}")
	public ResponseEntity<Need[]> getBasket(@PathVariable String username) {
		LOG.info("GET /basket/" + username);
		try {
			Integer[] basket = basketDao.getNeeds(username);
			ArrayList<Need> needs = new ArrayList<Need>();
			for (int i = 0; i < basket.length; i++) {
				needs.add(needDao.getNeed(basket[i]));
			}

			return new ResponseEntity<Need[]>(needs.toArray(new Need[needs.size()]), HttpStatus.OK);
		} catch (IOException e) {
			LOG.log(Level.SEVERE, e.getLocalizedMessage());
			return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}

	/**
	 * Deletes a {@link Need need} with the given id
	 *
	 * @param id The id of the {@link Need need} to be deleted
	 *
	 * @return ResponseEntity HTTP status of OK if deleted<br>
	 *         ResponseEntity with HTTP status of NOT_FOUND if not found<br>
	 *         ResponseEntity with HTTP status of INTERNAL_SERVER_ERROR otherwise
	 */
	@DeleteMapping("/{username}/{id}")
	public ResponseEntity<Need> removeNeed(@PathVariable String username, @PathVariable int id) {
		LOG.info("DELETE /basket/" + username + "/" + id);

		try {
			if (basketDao.removeNeed(username, id)) {
				Need need = needDao.getNeed(id);
				return new ResponseEntity<>(need, HttpStatus.OK);
			} else {
				return new ResponseEntity<>(HttpStatus.NOT_FOUND);
			}
		} catch (IOException e) {
			return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}

	@PostMapping("/{username}/checkout")
	public ResponseEntity<Integer> checkout(@PathVariable String username, @RequestBody Integer[] needIDs) {
		LOG.info("POST /basket/" + username + "/checkout " + needIDs);
		try {
			for (int i = 0; i < needIDs.length; i++) {
				// remove the need from the basket
				if (!basketDao.removeNeed(username, needIDs[i])) {
					return new ResponseEntity<Integer>(needIDs[i], HttpStatus.NOT_FOUND);
				}

				// update the need inventory
				Need need = needDao.getNeed(needIDs[i]);
				if (need == null) {
					return new ResponseEntity<Integer>(needIDs[i], HttpStatus.NOT_FOUND);
				}
				need.addToInventory(need.getAmountNeeded());
				needDao.updateNeed(need);
			}
			return new ResponseEntity<>(HttpStatus.OK);
		} catch (IOException e) {
			LOG.log(Level.SEVERE, e.getLocalizedMessage());
			return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}
}
